/*
 * Copyright (c) 2021, Marvell.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Marvell nor the
 *    names of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARVELL ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MARVELL BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __CAVIUM_PKCS11_VENDOR_DEFS_H__
#define __CAVIUM_PKCS11_VENDOR_DEFS_H__
#include "pkcs11.h"

/* HMAC KDF Mechanism */
#define CKM_SP800_108_COUNTER_KDF      (CKM_VENDOR_DEFINED | 0x00000001UL)

typedef struct CK_SP800_108_COUNTER_FORMAT {
    CK_ULONG   ulWidthInBits;
} CK_SP800_108_COUNTER_FORMAT;

typedef CK_SP800_108_COUNTER_FORMAT CK_PTR CK_SP800_108_COUNTER_FORMAT_PTR;

typedef struct CK_SP800_108_DKM_LENGTH_FORMAT {
    CK_ULONG  dkmLengthMethod;
    CK_ULONG  ulWidthInBits;
} CK_SP800_108_DKM_LENGTH_FORMAT;

typedef CK_SP800_108_DKM_LENGTH_FORMAT CK_PTR CK_SP800_108_DKM_LENGTH_FORMAT_PTR;

#define SP800_108_DKM_LENGTH_SUM_OF_KEYS 1
#define SP800_108_DKM_LENGTH_SUM_OF_SEGMENTS 2

typedef CK_ULONG CK_PRF_DATA_TYPE;
/* PRF data types */
#define CK_SP800_108_ITERATION_VARIABLE 0x0001
#define CK_SP800_108_DKM_LENGTH         0x0002
#define CK_SP800_108_BYTE_ARRAY         0x0003
#define SP800_108_COUNTER_FORMAT        0x0004
#define SP800_108_PRF_LABEL             0x0005
#define SP800_108_PRF_CONTEXT           0x0006
#define SP800_108_DKM_FORMAT            0x0007
#define SP800_108_BYTE_ARRAY            0x0008

typedef CK_MECHANISM_TYPE CK_PRF_TYPE;

typedef struct CK_PRF_DATA_PARAM {
    CK_PRF_DATA_TYPE   type;
    CK_VOID_PTR        pValue;
    CK_ULONG           ulValueLen;
} CK_PRF_DATA_PARAM;

typedef CK_PRF_DATA_PARAM CK_PTR CK_PRF_DATA_PARAM_PTR;

typedef struct CK_SP800_108_KDF_PARAMS {
    CK_PRF_TYPE            prftype;
    CK_ULONG               ulNumberOfDataParams;
    CK_PRF_DATA_PARAM_PTR  pDataParams;
} CK_SP800_108_KDF_PARAMS;


/* Note: CK_SP800_108_KDF_PARAMS will be sent as the mechanism parameter of
 * HMAC KDF mechanism (CKM_SP800_108_COUNTER_KDF).
 * prftype can be one of CKM_SHA_1_HMAC / SHA224_HMAC / SHA256_HMAC /
 * SHA384_HMAC / SHA512_HMAC.
 * Corrsponding to the mechanism, data has to be sent. This data will be stored
 * in a buffer pointed to by pDataParams. Each data param will be of tlv form
 * For eg. if we are using counter kdf mechanism (CKM_SP800_108_COUNTER_KDF),
 * we have to send counter format, prf label, prf context and dkm format in
 * the data param buffer. Eg. {SP800_108_COUNTER_FORMAT, (pointer to buff
 * containing CK_SP800_108_COUNTER_FORMAT), length of buffer},
 * {SP800_108_PRF_LABEL, (pointer to buff containing prf label), buffer length}
 * and so on.
 */

typedef struct CK_ATOMIC_KEY_GEN_AND_WRAP_KEY
{
    CK_MECHANISM_TYPE hashAlg;  // hashAlgo to be used for wrapping generated key
    CK_BYTE_PTR pWrappingKey;   // pointer to buffer containing RSA public key which is used to wrap key
    CK_ULONG ulWrappingKeyLen;  // length of rsa key data pointed by pWrappingKey
} CK_ATOMIC_KEY_GEN_AND_WRAP_KEY;

typedef CK_ATOMIC_KEY_GEN_AND_WRAP_KEY CK_PTR CK_ATOMIC_KEY_GEN_AND_WRAP_KEY_PTR;
#endif /* __CAVIUM_PKCS11_VENDOR_DEFS_H__ */
