/*
 * Copyright (c) 2021, Marvell.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Marvell nor the
 *    names of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARVELL ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MARVELL BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/* Don't include this file directly in the source code, instead include pkcs11_vendor.h.*/

#ifndef _PKCS11F_VENDOR_H_
#define _PKCS11F_VENDOR_H_
CK_VENDOR_PKCS11_FUNCTION_INFO(C_DualFALogin)
                (CK_SESSION_HANDLE hSession,        /* Session Handle.*/
                 CK_USER_TYPE userType,             /* Type of user, CU, CO and 2FA User.*/
                 CK_CHAR_PTR pPin,                  /* user PIN.*/
                 CK_ULONG ulPinLen,                 /* Length of the user PIN.*/
                 CK_ULONG portNum,                  /* Server port number for 2FA communication.*/
                 CK_CHAR_PTR pDualFAPrivateKey,     /* 2FA private key file name with path for SSL communication over the network.*/
                 CK_ULONG ulDualFAPrivateKeyLen,    /* Length of the 2FA private key file name with path.*/
                 CK_CHAR_PTR pDualFACertificate,    /* 2FA certificate file name with path for SSL communication over the network.*/
                 CK_ULONG ulDualFACertificateLen,   /* Lenght of the 2FA certificate with path.*/
                 CK_CHAR_PTR pDualFACipher,         /* SSL cipher to be user to communicate with 2FA client.*/
                 CK_ULONG ulDualFACipherLen);       /* Length of the SSL cipher string.*/

/*Description:
 * The user will pass pointers to get UDD previously set, the length of UDD and whether clear on use is set or not, on this thread.
 *    If pUDD as NULL & pointer to pUDDLen is not NULL, API will update the pUDDLen.
 *    If pUDD & pUDDLen are not NULL, then value in pUDDLen is compared with actual UDD length which was set earlier.
 *    If it is less than UDD length set, API will return CKR_BUFFER_TOO_SMALL & pUDDLen is updated with actual length.
 * Return values:
 *   CKR_OK, CKR_ARGUMENTS_BAD, CKR_BUFFER_TOO_SMALL.
 *   CKR_OK for success.
 *   CKR_ARGUMENTS_BAD if any invalid arguments passed like NULL pointer to the function.
 *   CKR_BUFFER_TOO_SMALL if length passed in pUDDLen is smaller than actual UDD length set.
 *
 *NOTE: Applications can call this API in multi-threaded environment.
 */
CK_VENDOR_PKCS11_FUNCTION_INFO(C_GetAuditUDD)
                (CK_CHAR_PTR pUDD,                  /* UDD data.*/
                 CK_ULONG_PTR pUDDLen,              /* Length of UDD data.*/
                 CK_BBOOL *pClearOnUse);            /* clear on use flag.*/

/*Description:
 * The user will pass pointer to the UDD to be set, the length of UDD and whether clear on use is set or not.
 * If bClearOnUse if TRUE, this UDD will be used only for the next API call.
 * If bClearOnUse if FALSE, UDD will be used for all the subsequent API calls, till this UDD is cleared with zero length UDD.
 * Input Parameter
 * pUDD : Pointer to UDD.
 * UlUDDLen : Length of UDD. Maximum supported UDD length is 32 bytes (MAX_AUDIT_UDD_LENGTH).
 * bClearOnUse : Boolean variable, to be used if UDD is to be cleared after operation. Default value is false.
 * Return values:
 *    CKR_OK, CKR_ARGUMENTS_BAD.
 *    CKR_OK for success.
 *    CKR_ARGUMENTS_BAD if any invalid arguments passed like NULL pointer to the function,
 *    if length passed is more than maximum UDD length.
 *NOTE: Applications can call this API in multi-threaded environment.
 */
CK_VENDOR_PKCS11_FUNCTION_INFO(C_SetAuditUDD)
                (CK_CHAR_PTR pUDD,                  /* UDD data.*/
                 CK_ULONG ulUDDLen,                 /* Length of UDD data.*/
                 CK_BBOOL bClearOnUse);             /* clear on use flag.*/


/*Description:
 * This API generates symmetric key and export the generated key which is wrapped using RSA public key.
 * hSession [in] session handle
 * pMechanism [in] pointer to CK_MECHANISM struct
 * pTemplate [in] pointer to attribute template of symmetric key
 * ulCount [in] number of attributes in pTemplate
 * pkcsArgs [in] pointer to CK_ATOMIC_KEY_GEN_AND_WRAP_KEY struct.
 *      All fields of the structure must be initialized.
 * pWrappedKey [out] pointer to buffer containing wrapped key data
 * pulWrappedKeyLen [out] length of wrapped key data pointed by pWrappedKey
 * phKey [out] key handle of generated symmetric key
 * */
CK_VENDOR_PKCS11_FUNCTION_INFO(C_AtomicGenAndWrapSymKey)
                (CK_SESSION_HANDLE hSession, 
                 CK_MECHANISM_PTR pMechanism,
                 CK_ATTRIBUTE_PTR pTemplate,
                 CK_ULONG ulCount,
                 CK_ATOMIC_KEY_GEN_AND_WRAP_KEY_PTR pkcsArgs, 
                 CK_BYTE_PTR pWrappedKey,
                 CK_ULONG_PTR pulWrappedKeyLen,
                 CK_OBJECT_HANDLE_PTR phKey);

/*Description:
 * This API generates asymmetric key pair and export the generated private key which is wrapped using RSA public key.
 * hSession [in] session handle
 * pMechanism [in] pointer to CK_MECHANISM struct
 * pPublicKeyTemplate [in] pointer to attribute template of public key
 * ulPublicKeyAttributeCount [in] number of attributes in pPublicKeyTemplate
 * pPrivateTemplate [in] pointer to attribute template of private key
 * ulPrivateKeyAttributeCount [in] number of attributes in pPrivateTemplate
 * pkcsArgs [in] pointer to CK_ATOMIC_KEY_GEN_AND_WRAP_KEY struct.
 *      All fields of the structure must be initialized.
 * pWrappedKey [out] pointer to buffer containing wrapped key data
 * pulWrappedKeyLen [out] length of wrapped key data pointed by pWrappedKey
 * phPublicKey [out] key handle of generated public key
 * phPrivateKey [out] key handle of generated private key
 * */
CK_VENDOR_PKCS11_FUNCTION_INFO(C_AtomicGenAndWrapASymKeyPair)
                (CK_SESSION_HANDLE hSession,
                 CK_MECHANISM_PTR pMechanism,
                 CK_ATTRIBUTE_PTR pPublicKeyTemplate,
                 CK_ULONG ulPublicKeyAttributeCount,
                 CK_ATTRIBUTE_PTR pPrivateTemplate,
                 CK_ULONG ulPrivateKeyAttributeCount,
                 CK_ATOMIC_KEY_GEN_AND_WRAP_KEY_PTR pkcsArgs, 
                 CK_BYTE_PTR pWrappedKey,
                 CK_ULONG_PTR pulWrappedKeyLen,
                 CK_OBJECT_HANDLE_PTR phPublicKey,
                 CK_OBJECT_HANDLE_PTR phPrivateKey);
#endif /* _PKCS11F_VENDOR_H_ */
